/*************************************************************************
 *
 * Autor:       Ales Povalac
 * Kontakt:     <alpov@alpov.net>
 *
 * Popis:       obsluha 1-wire sbernice
 *
 *************************************************************************
 *
 * $Id: 1wire.c 95 2008-03-19 13:21:26Z alpov $
 * 
 *************************************************************************/         


/*************************************************************************
 *
 * obsluha 1-wire sbernice
 * 
 * (c) Ales Povalac, alpov@alpov.net, http://www.alpov.net/
 * 
 *************************************************************************
 *
 * *Id: 1wire.c 123 2007-02-18 20:35:06Z alpov *
 * 
 *************************************************************************/         

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <util/crc16.h>
#include "1wire.h"


/******************************************************************
 * LOW LEVEL funkce pro 1-wire sbernici
 ******************************************************************/ 

void OWInit(void)
{
    OWSetLink(1);
    OWInitHw();         // nastaveni DDR
    
    OWReset();
}

unsigned char OWReset(void)
{
    unsigned char result;
    unsigned char sreg = SREG;
    
    DELAY_G;
    cli();
    OWSetLink(0);
    DELAY_H;
    OWSetLink(1);
    DELAY_I;
    result = OWGetLink();
    SREG = sreg;
    DELAY_J;
    return result;
}

void OWWriteBit(unsigned char bit)
{
    unsigned char sreg = SREG;
    
    cli();
    if (bit) {
        OWSetLink(0);
        DELAY_A;
        OWSetLink(1);
        DELAY_B;
    } else {
        OWSetLink(0);
        DELAY_C;
        OWSetLink(1);
        DELAY_D;
    }
    SREG = sreg;
}

unsigned char OWReadBit(void)
{
    unsigned char result;
    unsigned char sreg = SREG;

    cli();
    OWSetLink(0);
    DELAY_A;
    OWSetLink(1);
    DELAY_E;
    result = OWGetLink();
    SREG = sreg;
    DELAY_F;
    return result;
}

void OWWriteByte(unsigned char data)
{
    unsigned char i;
    
    for (i = 0; i < 8; i++) {
        OWWriteBit(data & 0x01);
        data >>= 1;
    }
}

unsigned char OWReadByte(void)
{
    int i, result = 0;
    
    for (i = 0; i < 8; i++) {
        result >>= 1;
        if (OWReadBit())
            result |= 0x80;
    }
    return result;
}

void OWSendCmd(unsigned char *ROM, unsigned char cmd)
{
    unsigned char i;

    OWReset();
    if (ROM == NULL) {
        OWWriteByte(OW_SKIP_ROM_CMD);
    } else {
        OWWriteByte(OW_MATCH_ROM_CMD);
        for (i = 0; i < 8; i++)
            OWWriteByte(ROM[i]);
    }
    OWWriteByte(cmd);
}

void OWCRC(unsigned char x, unsigned char *crc)
{
    *crc = _crc_ibutton_update(*crc, x);
}

/******************************************************************
 * HIGH LEVEL funkce pro 1-wire sbernici
 ******************************************************************/ 

void OWConvertAll(void)
{
    OWSendCmd(NULL, OW_CONVERT_T_CMD);
}

unsigned char OWReadTemperature(int *result)
{       
    unsigned char i, crc = 0, buf[8];
    
    OWSendCmd(NULL, OW_RD_SCR_CMD);
    for (i = 0; i < 8; i++) {
        buf[i] = OWReadByte();
        OWCRC(buf[i], &crc);
    }
    if (crc != OWReadByte()) {
        *result = OW_ERR_BADCRC;
        return 0;
    }

    #if (SENSOR_TYPE == 1)
            *result = (signed char)((buf[1] & 0x80) | (buf[0] >> 1)) * 100 + 75 - buf[6] * 100 / 16;
    
    #elif (SENSOR_TYPE == 2)            
            *result = (signed char)((buf[1] << 4) | (buf[0] >> 4)) * 100 + (buf[0] & 0x0F) * 100 / 16;
    
    #else
            *result = OW_ERR_BADFAMILY;
            return 0;
    #endif
    
    return 1;
}

