/*
 * meteo_tx.c
 *
 * vysilaci modul meteostanice
 *
 * Ji Dvok, 5.2010 
 */    

#include <avr/io.h>
#include <avr/interrupt.h> 
#include <util/delay.h>
#include <avr/sleep.h>
#include "SHT.h"
#include "RFM12B.h"

/**************MACROS*********************************************/

#define LED_G_OFF() PORTC &= ~(1<<4)
#define LED_G_ON() PORTC |= (1<<4)
#define LED_Y_OFF() PORTC &= ~(1<<3)
#define LED_Y_ON() PORTC |= (1<<3)
#define LED_R_OFF() PORTC &= ~(1<<2)
#define LED_R_ON() PORTC |= (1<<2)


/**************VARIABLES*********************************************/
volatile int j;


/**************INTERRUPTS*********************************************/
ISR(TIMER2_OVF_vect)
{
}

/**************FUNCTIONS*********************************************/
void sleepInit(void) //inicializace registr pro sleep mod (inc t/c)
{
ASSR |= (1<<AS2);                            //asynchronni, ext. krystal
TCCR2 = (1<<CS22) | (1<<CS21) | (1<<CS20);   //preddelicka 1024
TCNT2 = 0;                                   //nulovani
TIFR |= (1<<TOV2);                           //nulovani priznaku preteceni
TIMSK |= (1<<TOIE2);  
j=0;                                         //povoleni prerus. preteceni T/C2

set_sleep_mode(SLEEP_MODE_PWR_SAVE);         // nastaveni sleep modu
sei();                                       // povoleni global. preruseni
}


/**************MAIN*********************************************/

int main() {
  volatile float humi_val, temp_val, dew;
  char buffer1 [5];
  char buffer2 [5];
  char buffer3 [5];
  
  portInit();
  rfInit();
  sleepInit();
  _delay_ms(500);
  while(1)
  {
   
  if(j==5)
    {
    
    LED_G_ON();
    SHT_mer(&humi_val,&temp_val);
    dew = calc_dewpoint(humi_val,temp_val);
    
    sprintf(buffer1,"%2.1f",temp_val);
    sprintf(buffer2,"%2.1f",humi_val);
    sprintf(buffer3,"%2.1f",dew);
    
    LED_Y_ON();
    
    writeCmd(0x0000);
    writeCmd(0x8239);
    rfSend(0xAA); // PREAMBLE
    rfSend(0xAA);
    rfSend(0xAA);
    rfSend(0x2D); // SYNC
    rfSend(0xD4);
    
    
    rfSend(buffer1[0]);
    rfSend(buffer1[1]);
    rfSend(buffer1[2]);
    rfSend(buffer1[3]);
    rfSend(buffer1[4]);

    rfSend(buffer2[0]);
    rfSend(buffer2[1]);
    rfSend(buffer2[2]);
    rfSend(buffer2[3]);
    rfSend(buffer2[4]);
    
    rfSend(buffer3[0]);
    rfSend(buffer3[1]);
    rfSend(buffer3[2]);
    rfSend(buffer3[3]);
    rfSend(buffer3[4]);
    
    rfSend(0xAA); // DUMMY BYTES
    rfSend(0xAA);
    rfSend(0xAA);
    writeCmd(0x8201);
    LED_Y_OFF();
    j=0;
    LED_G_OFF();
    }
    else
    {
    sleep_mode();
    j++;
    };
  };
}


