/***********************************************************************************
Project:          SHTxx demo program (V2.1)
Filename:         SHTxx_Sample_Code.c    

Prozessor:        80C51 family
Compiler:         Keil Version 6.14

Autor:            MST
Copyrigth:        (c) Sensirion AG

Upraveno pro meteostanici: Jiri Dvorak 
Datum: 1.3.2010     
***********************************************************************************/
#include <math.h>    //Keil library  
#include <stdio.h>   //Keil library
#include <avr/io.h>
#include <util/delay.h>
#include "SHT.h"
//----------------------------------------------------------------------------------
// modul-var
//----------------------------------------------------------------------------------
enum {TEMP,HUMI};




int temp;
float napeti_tlakomeru;
float Vref=4.7; // nastaveno referenci TL431, AREF pin
float Vs=5.0;   // nastaveno referenci TL431, napajeni cidla MPX6115
float offset=-5.0; // hodnota o kterou se lisil udaj absolutniho tlaku

float T;
float nadmorska_vyska=300.0;  // nadmorska vyska stanice
float tlak_abs;
float tlak_0m;

volatile unsigned int cas=0; //promenna pro sleep mode


//------------------------------------------------------------------------------

// KOD PRO MERENI Z SHT----
char s_write_byte(unsigned char value)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge 
{ 
  unsigned char i,error=0;  
  for (i=0x80;i>0;i/=2)             //shift bit for masking
  { if (i & value) DATA_w1          //masking value with i , write to SENSI-BUS
    else DATA_w0;
    SCK_w1;                         //clk for SENSI-BUS
    _delay_us(5);
    SCK_w0;
    _delay_us(5);
  }
  DATA_w1;                          //release DATA-line
  SCK_w1;                           //clk #9 for ack
  _delay_us(5);
  error=DATA_r;                     //check ack (DATA will be pulled down by SHT11)
  SCK_w0;
  _delay_us(5);
  return error;                     //error=1 in case of no acknowledge
}

//----------------------------------------------------------------------------------
char s_read_byte(unsigned char ack)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1" 
{ 
  unsigned char i,val=0;
  DATA_w1;                          //release DATA-line
  _delay_us(5);
  for (i=0x80;i>0;i/=2)             //shift bit for masking
  { SCK_w1;                         //clk for SENSI-BUS
    _delay_us(5);
    if (DATA_r) val=(val | i);      //read bit
    SCK_w0;
    _delay_us(5);
  }
  if (!ack) DATA_w1 else DATA_w0;   //in case of "ack==1" pull down DATA-Line
  SCK_w1;                           //clk #9 for ack
  _delay_us(5);                     //pulswith approx. 5 us
  SCK_w0;
  DATA_w1;                          //release DATA-line
  _delay_us(5);
  return val;
}

//----------------------------------------------------------------------------------
void s_transstart(void)
//----------------------------------------------------------------------------------
// generates a transmission start 
//       _____         ________
// DATA:      |_______|
//           ___     ___
// SCK : ___|   |___|   |______
{  
   DATA_w1; SCK_w0;                 //Initial state
   _delay_us(5);
   SCK_w1;
   _delay_us(5);
   DATA_w0;
   _delay_us(5);
   SCK_w0;
   _delay_us(5);
   SCK_w1;
   _delay_us(5);
   DATA_w1;
   _delay_us(5);
   SCK_w0;
}

//----------------------------------------------------------------------------------
void s_connectionreset(void)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
{  
  unsigned char i; 
  DATA_w1; SCK_w0;                  //Initial state
  _delay_us(5);
  for(i=0;i<9;i++)                  //9 SCK cycles
  { SCK_w1;
    _delay_us(5);
    SCK_w0;
    _delay_us(5);
  }
  s_transstart();                   //transmission start
}

//----------------------------------------------------------------------------------
char s_softreset(void)
//----------------------------------------------------------------------------------
// resets the sensor by a softreset 
{ 
  unsigned char error=0;  
  s_connectionreset();              //reset communication
  error+=s_write_byte(RESET);       //send RESET-command to sensor
  return error;                     //error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
char s_read_statusreg(unsigned char *p_value, unsigned char *p_checksum)
//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
{ 
  unsigned char error=0;
  s_transstart();                   //transmission start
  error=s_write_byte(STATUS_REG_R); //send command to sensor
  *p_value=s_read_byte(ACK);        //read status register (8-bit)
  *p_checksum=s_read_byte(noACK);   //read checksum (8-bit)  
  return error;                     //error=1 in case of no response form the sensor
}

//----------------------------------------------------------------------------------
char s_write_statusreg(unsigned char *p_value)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{ 
  unsigned char error=0;
  s_transstart();                   //transmission start
  error+=s_write_byte(STATUS_REG_W);//send command to sensor
  error+=s_write_byte(*p_value);    //send value of status register
  return error;                     //error>=1 in case of no response form the sensor
}
 							   
//----------------------------------------------------------------------------------
char s_measure(unsigned char *p_value, unsigned char *p_checksum, unsigned char mode)
//----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
{ 
  unsigned char error=0;
  unsigned int i;

  s_transstart();                   //transmission start
  switch(mode){                     //send command to sensor
    case TEMP	: error+=s_write_byte(MEASURE_TEMP); break;
    case HUMI	: error+=s_write_byte(MEASURE_HUMI); break;
    default     : break;	 
  }
  for (i=0;i<2000;i++) {
    _delay_ms(1);
    if(DATA_r==0) break;
  } //wait until sensor has finished the measurement
  if(DATA_r) error+=1;              // or timeout (~2 sec.) is reached
  
    *(p_value+1)=s_read_byte(ACK);    //read the first byte (MSB)
  *(p_value)  =s_read_byte(ACK);    //read the second byte (LSB)
  *p_checksum =s_read_byte(noACK);  //read checksum
 
  
  
  
  return error;
}

//----------------------------------------------------------------------------------------
void calc_sth11(float *p_humidity ,float *p_temperature)
//----------------------------------------------------------------------------------------
// calculates temperature [C] and humidity [%RH] 
// input :  humi [Ticks] (12 bit) 
//          temp [Ticks] (14 bit)
// output:  humi [%RH]
//          temp [C]
{ const float C1=-4.0;              // for 12 Bit
  const float C2=+0.0405;           // for 12 Bit
  const float C3=-0.0000028;        // for 12 Bit
  const float T1=+0.01;             // for 14 Bit @ 5V
  const float T2=+0.00008;           // for 14 Bit @ 5V	

  float rh=*p_humidity;             // rh:      Humidity [Ticks] 12 Bit 
  float t=*p_temperature;           // t:       Temperature [Ticks] 14 Bit
  float rh_lin;                     // rh_lin:  Humidity linear
  float rh_true;                    // rh_true: Temperature compensated humidity
  float t_C;                        // t_C   :  Temperature [C]

  t_C=t*0.01 - 40;                  //calc. temperature from ticks to [C]
  rh_lin=C3*rh*rh + C2*rh + C1;     //calc. humidity from ticks to [%RH]
  rh_true=(t_C-25)*(T1+T2*rh)+rh_lin;   //calc. temperature compensated humidity [%RH]
  if(rh_true>100)rh_true=100;       //cut if the value is outside of
  if(rh_true<0.1)rh_true=0.1;       //the physical possible range

  *p_temperature=t_C;               //return temperature [C]
  *p_humidity=rh_true;              //return humidity[%RH]
}

//--------------------------------------------------------------------
float calc_dewpoint(float h,float t)
//--------------------------------------------------------------------
// calculates dew point
// input:   humidity [%RH], temperature [C]
// output:  dew point [C]
{ float logEx,dew_point;
  logEx=0.66077+7.5*t/(237.3+t)+(log10(h)-2);
  dew_point = (logEx - 0.66077)*237.3/(0.66077+7.5-logEx);
  return dew_point;
}

//----------------------------------------------------------------------------------
void SHT_mer(float *p_humidity ,float *p_temperature)
{

   unsigned int i_humi_val, i_temp_val;
    float humi_val, temp_val;
    unsigned char error,checksum;
    error=0;
    error+=s_measure((unsigned char*) &i_humi_val,&checksum,HUMI);  
    error+=s_measure((unsigned char*) &i_temp_val,&checksum,TEMP); 
    if(error!=0) 
    {s_connectionreset();
    //LED_R_ON();
    return 0;
    }                //in case of an error: connection reset
    else
    { 
      humi_val=(float)i_humi_val;   //converts integer to float
      temp_val=(float)i_temp_val;   //converts integer to float
      calc_sth11(&humi_val,&temp_val);  //calculate humidity, temperature
      *p_temperature = temp_val;
      *p_humidity = humi_val;      
     
      
            
    };
}
//----------------------------------------------------------------------------------
// sample program that shows how to use SHT11 functions
// 1. connection reset 
// 2. measure humidity [ticks](12 bit) and temperature [ticks](14 bit)
// 3. calculate humidity [%RH] and temperature [C]
// 4. calculate dew point [C]
// 5. print temperature, humidity, dew point  


